#!/usr/bin/env bash
#
# Copyright (C) 2011-2014 Entware
#
# This is free software, licensed under the GNU General Public License v2.
# See /LICENSE for more information.
#

function index {
    pkg_dir=.
    for pkg in `ls -1 $pkg_dir/*.ipk | sort`; do
	name="${pkg##*/}"
	name="${name%%_*}"
	[[ "$name" = "kernel" ]] && continue
	echo "Generating index for package $pkg" >&2
	file_size=$(ls -l $pkg | awk '{print $5}')
	md5sum=$(md5sum $pkg | awk '{print $1}')
	sha256sum=$(openssl sha256 $pkg | awk '{print $2}')
	# Take pains to make variable value sed-safe
	sed_safe_pkg=`echo $pkg | sed -e 's/^\.\///g' -e 's/\\//\\\\\\//g'`
	tar -xzOf $pkg ./control.tar.gz | tar xzOf - ./control | sed -e "s/^Description:/Filename: $sed_safe_pkg\\
Size: $file_size\\
MD5Sum: $md5sum\\
SHA256sum: $sha256sum\\
Description:/"
	echo ""
    done
}

###################

BASE_DIR=/usr/share/nginx/entware
# Some measures on this VPS:
# indexing 5000 packages takes ~2,5 minutes,
# archiving ~0,5 minutes.

for feed in {entware,mipselsf,entware_test}
do
    cd $BASE_DIR/binaries/$feed
    # Skip feed if no packages here
    [ -z "$(ls *.ipk)" ] &&  continue
    # Index feed only if there is no index or it's older than any of packages
    if [ ! -f Packages.gz ] || [ -n "$(find -maxdepth 1 -type f -name "*.ipk" -newer Packages.gz)" ]
    then
	# Delete old temp index file older then 30 minutes. Something went wrong.
	[ -n "$(find -name "Packages.tmp" -mtime +30)" ] && rm -f Packages.tmp
	# Previously started indexer is still working, skipping this feed
	[ -f Packages.tmp ] && continue

	echo "Indexing $feed feed..."
	index > Packages.tmp
	# This is a trick for instant replacing feed index
	# Otherwise, index will be absent for ~2,5 minutes
	mv -f Packages.tmp Packages
	gzip -9c Packages > Packages.gz

	/usr/local/bin/ipk_indexer_html_sorted.sh > Packages.html

	echo "Archiving $feed feed..."
	[ -d archive ] || mkdir archive
	cp -u *.ipk archive
    fi
done
